﻿/**
 * Written by Peter Backx as part of a tutorial series on using YouTube in ActionScript 3.0
 * Please see http://www.streamhead.com for more information
 */
package 
{
	import com.flashdynamix.events.CustomEvent;
	import com.flashdynamix.services.YouTube;
	import flash.display.Sprite;
	import flash.events.AsyncErrorEvent;
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.NetStatusEvent;
	import flash.events.SecurityErrorEvent;
	import flash.media.Video;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	
	public class Main extends Sprite {
		private var connection : NetConnection;
		private var stream : NetStream;
		private var video : Video;
		private var yt : YouTube;

		public function Main():void {
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void {
			removeEventListener(Event.ADDED_TO_STAGE, init);

			trace("creating NetConnection");
			connection = new NetConnection();
			connection.addEventListener(NetStatusEvent.NET_STATUS, netStatusHandler);
			connection.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
			connection.connect(null);
		}
		
		private function netStatusHandler(event : NetStatusEvent):void {
            switch (event.info.code) {
                case "NetConnection.Connect.Success":
                    connectStream();
                    break;
                case "NetStream.Play.StreamNotFound":
                    trace("Unable to locate video");
                    break;
				case "NetStream.Play.Start":
					trace("start playing");
					break;
				case "NetStream.Play.Stop" :
					trace("stop playing");
					break;
			}
		}
		
        private function securityErrorHandler(event:SecurityErrorEvent):void {
            trace("securityErrorHandler: " + event);
        }
		
		private function connectStream():void {
			trace("start YouTube search");
			yt = new YouTube();
			yt.addEventListener(Event.COMPLETE, onLoaded);
			yt.addEventListener(ErrorEvent.ERROR, onError);
			yt.videosForTag("lego");
		}
			
		private function onLoaded(e:CustomEvent):void {
			switch (e.id) {
				case YouTube.SEARCH :
					trace("search finished");
					try {
						trace("first item : " + e.value.items[0].title + " - " + e.value.items[0].link);
					} catch (evt:ArgumentError) {
						trace("ERROR : No Videos For Tag");
					}
					trace("requesting the video id and t-parameter");
					yt.videoId(e.value.items[0].link);
					break;
				case YouTube.VIDEOID :
					trace("loaded: video id = " + e.value.id + " and T-parameter = " + e.value.t);
					play(YouTube.FLVUrl + e.value.id + "&t=" + e.value.t);
					break;
			}
		}
		
		private function onError(e:ErrorEvent):void {
			trace("IOError : " + e );
		}
	
		private function play(url:String):void {
			trace("playing: " + url);
            stream = new NetStream(connection);
            stream.addEventListener(NetStatusEvent.NET_STATUS, netStatusHandler);
            stream.addEventListener(AsyncErrorEvent.ASYNC_ERROR, asyncErrorHandler);
            video = new Video();
            video.attachNetStream(stream);
            stream.play(url);
            addChild(video);
		}
	
        private function asyncErrorHandler(event:AsyncErrorEvent):void {
            // ignore AsyncErrorEvent events.
        }
	}
	
}