package com.flashdynamix.controls {
	import flash.display.*;
	import flash.events.*;
	import flash.media.Video;
	import flash.utils.Timer;
	
	import com.flashdynamix.effects.Tweener;
	import com.flashdynamix.events.CustomEvent;
	import com.flashdynamix.events.VideoEvent;
	import com.flashdynamix.net.FLVPlayer;
	import com.flashdynamix.utils.NumberUtil;	

	public class VideoController extends Sprite {

		public var playback : VideoPlayback;
		public var playAgainPB : MovieClip;
		public var myVid : Video;
		public var poller : MovieClip;

		private var fvp : FLVPlayer;
		private var counter : Timer;
		private var inActive : Timer;
		private var tw : Tweener;
		private var playBackHidden : Boolean = false;

		public function VideoController() {
			fvp = new FLVPlayer(myVid);
			counter = new Timer(100);
			inActive = new Timer(8 * 1000, 1);
			tw = new Tweener();
			
			playAgainPB.visible = false;
			poller.visible = false;			
			playAgainPB.buttonMode = true;
			poller.mouseEnabled = false;
			
			poller.addEventListener(Event.ENTER_FRAME, function():void {
				poller.arrow.rotation -= 15;
			});
			counter.addEventListener(TimerEvent.TIMER, onTimer, false, 0, true);
			inActive.addEventListener(TimerEvent.TIMER, lazyMouse, false, 0, true);
			playback.addEventListener(Event.CHANGE, onClick, false, 0, true);
			fvp.addEventListener(Event.CHANGE, onChange, false, 0, true);
			fvp.addEventListener(Event.COMPLETE, onComplete, false, 0, true);
			playAgainPB.addEventListener(MouseEvent.CLICK, onClick, false, 0, true);
			addEventListener(MouseEvent.MOUSE_MOVE, move, false, 0, true);
					
			counter.start();
		}

		private function lazyMouse(e : TimerEvent) : void {
			if(!fvp.buffering) {
				playBackHidden = true;
				tw.remove(playback);
				tw.add(playback, {y:240}, 4);
			} else {
				inActive.start();
			}
		}

		private function move(e : MouseEvent) : void {
			inActive.start();
			if(playBackHidden) {
				tw.remove(playback);
				tw.add(playback, {y:218}, 1);
				playBackHidden = false;
			}
		}

		public function play(url : String) : void {
			playAgainPB.visible = false;
			poller.visible = true;
			fvp.load(url);
			inActive.start();
		}

		public function stop() : void {
			fvp.stop();
		}

		public function set volume(num : Number) : void {
			fvp.volume = num;
			playback.volume = num;
		}

		public function close() : void {
			fvp.stop();
			fvp.close();
		}

		public function dispose() : void {
			removeEventListener(MouseEvent.MOUSE_MOVE, move);
			counter.removeEventListener(TimerEvent.TIMER, onTimer);
			inActive.removeEventListener(TimerEvent.TIMER, onTimer);
			playback.removeEventListener(Event.CHANGE, onClick);
			fvp.removeEventListener(Event.CHANGE, onChange);
			fvp.removeEventListener(Event.COMPLETE, onComplete);
			playAgainPB.removeEventListener(MouseEvent.CLICK, onClick);
			
			inActive.stop();
			counter.stop();
			
			tw.dispose();
			fvp.dispose();
		}

		private function onTimer(e : TimerEvent) : void {
			playback.loaded = fvp.percentLoaded;
			playback.progress = fvp.progress;
			playback.info = NumberUtil.time(fvp.time).time;
		}

		private function onChange(e : CustomEvent) : void {
			poller.visible = fvp.buffering;
			fvp.pause(!fvp.buffering);
			
			if(fvp.buffering) {
				if(playBackHidden) {
					inActive.start();
					tw.remove(playback);
					tw.add(playback, {y:218}, 1);
					playBackHidden = false;
				}
			}
		}

		private function onComplete(e : CustomEvent) : void {
			poller.visible = false;
			playAgainPB.visible = true;
			dispatchEvent(new CustomEvent(Event.COMPLETE));
		}

		private function onClick(e : *) : void {
			var ct : * = e.currentTarget;
			switch(ct) {
				case playback :
					switch (e.id) {
						case VideoEvent.REWIND :
							fvp.rewind();
							playAgainPB.visible = false;
							break;
						case VideoEvent.PAUSE :
							fvp.pause();
							break;
						case VideoEvent.PLAY :
							fvp.pause();
							break;
						case VideoEvent.SOUND :
							fvp.volume = e.value;
							break;
						case VideoEvent.SEEK :
							fvp.seek(e.value);
							playAgainPB.visible = false;
							break;
					}
					break;
				case playAgainPB :
					playAgainPB.visible = false;
					fvp.rewind();
					break;
			}
		}
	}
}