package com.flashdynamix.controls {
	import flash.display.*;
	import flash.events.*;
	import flash.text.TextField;

	import com.flashdynamix.events.CustomEvent;
	import com.flashdynamix.events.VideoEvent;	

	public class VideoPlayback extends Sprite {

		public var barHolder : MovieClip;
		public var rewindPB : MovieClip;
		public var pausePB : MovieClip;
		public var playPB : MovieClip;
		public var soundTogglePB : MovieClip;
		public var infoTxtBx : TextField;
		public var poller : MovieClip;

		private var _soundOn : Boolean = true;
		private var progressBar : MovieClip;
		private var loadBar : MovieClip;
		private var soundThumb : MovieClip;
		private var soundTrack : MovieClip;

		public var allowSeek : Boolean = true;

		public function VideoPlayback() {

			progressBar = barHolder.progress;
			loadBar = barHolder.loaded;

			if(allowSeek) {
				progressBar.mouseEnabled = false;
				loadBar.buttonMode = true;
				loadBar.addEventListener(MouseEvent.MOUSE_UP, onClick);
			}
			
			addButton(rewindPB);
			addButton(pausePB);
			addButton(playPB);
			addButton(soundTogglePB);
			
			playPB.alpha = 1;
			playPB.visible = false;
		}

		private function addButton(item : MovieClip) : void {
			item.buttonMode = true;
			item.addEventListener(MouseEvent.MOUSE_UP, onClick);
		}

		private function onClick(e : MouseEvent) : void {
			var evt : CustomEvent = new CustomEvent(Event.CHANGE);
			switch (e.target) {
				case rewindPB :
					evt.id = VideoEvent.REWIND;
					break;
				case pausePB :
					evt.id = VideoEvent.PAUSE;
					pausePB.visible = false;
					playPB.visible = true;
					break;
				case playPB :
					evt.id = VideoEvent.PAUSE;
					pausePB.visible = true;
					playPB.visible = false;
					break;
				case soundTogglePB :
					_soundOn = !soundOn;
					if (soundOn) {
						evt.value = 1;
						soundTogglePB.alpha = 1;
					} else {
						evt.value = 0;
						soundTogglePB.alpha = 0.5;
					}
					evt.id = VideoEvent.SOUND;
					
					break;
				case loadBar :
					evt.id = VideoEvent.SEEK;
					var x : Number = loadBar.mouseX * loadBar.scaleX;
					evt.value = Math.min(1, (x / 205));
					break;
			}
			dispatchEvent(evt);
		}

		private function mouseUp(e : MouseEvent = null) : void {
			soundThumb.removeEventListener(MouseEvent.MOUSE_UP, mouseUp);
			
			soundThumb.stopDrag();
		}

		public function get soundOn() : Boolean {
			return _soundOn;
		}

		public function set loaded(num : Number) : void {
			loadBar.scaleX = num;
		}

		public function set progress(num : Number) : void {
			progressBar.scaleX = num;
		}

		public function set volume(per : Number) : void {
			var range : Number = (soundTrack.height - soundThumb.height);
			soundThumb.y = soundTrack.y + (range * (1 - per));
		}

		public function get volume() : Number {
			var range : Number = (soundTrack.height - soundThumb.height);
			var per : Number = (soundThumb.y - soundTrack.y) / range;
			
			return 1 - per;
		}

		public function set info(txt : String) : void {
			infoTxtBx.text = txt;
		}
	}
}