﻿package com.flashdynamix.effects {
	import flash.display.Sprite;
	import flash.events.*;
	
	import com.flashdynamix.events.CustomEvent;
	
	import fl.motion.easing.Quintic;	

	/**
	 * @author Shane McCartney
	 */
	public class Tweener extends EventDispatcher{
		
		private var items:Array;
		private var frame:Sprite;
		
		public function Tweener(){
		}
		public function add(instance:*, target:Object, duration:Number = 0.5, tween:Function = null, delay:Number = 0, data:* = null):*{
			if(instance == null) return;
			
			if(!hasItems) {
				frame = new Sprite();
				items = new Array();
				frame.addEventListener(Event.ENTER_FRAME, update);
			}
			
			if(tween == null) tween = Quintic.easeOut;
			
			if(instance is Array){
				var tweens:Array = new Array();
				for(var i:int =0; i<instance.length; i++){
					tweens.push(new TweenItem(instance[i], target, duration, tween, delay, data));
				}
				items = items.concat(tweens);
				
				return tweens;
			} else {
				
				var ti:TweenItem = new TweenItem(instance, target, duration, tween, delay, data);
				items.push(ti);
				
				return ti;
			}
		}
		public function removeAll():void{
			if(!hasItems) return;
			
			for (var i:int = 0; i<items.length; i++){
				removeAtIndex(i);
				i--;
			}
		}
		public function remove(instance:*):Boolean {
			if(!hasItems) return false;
			
			var hasRemoved:Boolean = false;
			for (var i:int = 0; i<items.length; i++){
				var ti:TweenItem = items[i];
				if(ti.instance == instance){
					removeAtIndex(i);
					i--;
					hasRemoved = true;
				}
				
			}
			return hasRemoved;
		}
		public function removeByData(data:*):Boolean{
			if(!hasItems) return false;
			
			var hasRemoved:Boolean = false;
			for (var i:int = 0; i<items.length; i++){
				var ti:TweenItem = items[i];
				if(ti.data == data){
					removeAtIndex(i);
					i--;
					hasRemoved = true;
				}
			}
			return hasRemoved;
		}
		public function getItem(instance:*):TweenItem{
			for (var i:int = 0; i<items.length; i++){
				var ti:TweenItem = items[i];
				if(ti.instance == instance){
					return ti;
				}
			}
			return null;
		}
		public function dispose():void{
			if(items == null) return;
			
			removeAll();
			items = null;
			frame.removeEventListener(Event.ENTER_FRAME, update);
			frame = null;
		}
		private function removeAtIndex(idx:int):void{
			items[idx] = null;
			items.splice(idx, 1);
		}
		private function update(e:Event):void{
			if(!hasItems) return;
			
			var i:int = 0;
			while (i<items.length){
				var ti:TweenItem = items[i];
				dispatchEvent( new CustomEvent(Event.CHANGE, null, ti));
				if(ti.isPlaying){
					ti.update();
				} else if(ti.finished){
					ti.update();
					removeAtIndex(i);
					i--;
					if(!hasItems) {
						removeAll();
					}
				} 
				i++;
			} 
			dispatchEvent( new CustomEvent(Event.ENTER_FRAME));
			if(!hasItems) {
				dispatchEvent( new CustomEvent(Event.COMPLETE) );
			}
		}
		public function get length():int{
			if(items == null) return 0;
			return items.length;
		}
		public function get hasItems():Boolean{
			return (length>0); 
		}
		public static function ease(target:Number, current:Number):Number{
			return current + (target-current)/10;
		}
	}
}