package com.flashdynamix.net {
	import flash.events.*;
	import flash.net.*;
	
	import com.flashdynamix.events.CustomEvent;	

	public class DataLoader extends EventDispatcher{
		
		private var requests:Array;
		private var _current:Number = 0;
		public var dataFormat:String = URLLoaderDataFormat.TEXT;
		
		public function DataLoader(){
			requests = new Array();
		}
		public function load(lr:URLRequest, id:* = null):URLLoader{
			var ul:URLLoader = new URLLoader(lr);
			ul.dataFormat = dataFormat;
			
			requests.push( new DataItem(id, ul, lr) );
			
			ul.addEventListener(Event.COMPLETE, onComplete, false, 0, true);
			ul.addEventListener(ProgressEvent.PROGRESS, onProgress, false, 0, true);
			ul.addEventListener(IOErrorEvent.IO_ERROR, onError, false, 0, true);
			
			ul.load(lr);
			
			_current++;
			
			return ul;
		}
		public function closeAll():void{
			for (var i:int =0; i<requests.length; i++){
				removeItem(i);
			}
		}
		public function getItemById(id:*):DataItem{
			for (var i:int =0; i<requests.length; i++){
				var di:DataItem = requests[i];
				if(di.id == id){
					return di;
				}
			}
			return null;	
		}
		public function getItemByLoader(loader:URLLoader):DataItem{
			for (var i:int =0; i<requests.length; i++){
				var di:DataItem = requests[i];
				if(di.loader == loader){
					return di;
				}
			}
			return null;	
		}
		public function has(id:*):Boolean {
			if(getItemById(id) == null) return false;
			return true;
		}
		private function removeItem(index:int):void{
			var di:DataItem = requests[index];
			var ul:URLLoader = di.loader;
			
			if(di.loading){
				ul.close();
				_current--;
			}
			
			ul.removeEventListener(Event.COMPLETE, onComplete);
			ul.removeEventListener(ProgressEvent.PROGRESS, onProgress);
			ul.removeEventListener(IOErrorEvent.IO_ERROR, onError);
			
			requests[index] = null;
			requests.splice(index, 1);
		}
		public function dispose():void{
			closeAll();
			requests = null;
		}
		private function onProgress(e:ProgressEvent):void{
			var ul:URLLoader = e.target as URLLoader;
			var item:DataItem = getItemByLoader(ul);		
			dispatchEvent(new CustomEvent(ProgressEvent.PROGRESS, item.id, item));
		}
		private function onComplete(e:Event):void{
			var ul:URLLoader = e.target as URLLoader;
			var item:DataItem = getItemByLoader(ul);
			item.data = ul.data;
			_current--;
			dispatchEvent(new CustomEvent(Event.COMPLETE, item.id, item));
		}
		private function onError(e:IOErrorEvent):void{
			var ul:URLLoader = e.target as URLLoader;
			var item:DataItem = getItemByLoader(ul);
			_current--;
			dispatchEvent(new CustomEvent(ErrorEvent.ERROR, item.id, item));
		}
		public function get bytesLoaded():Number{
			var total:Number = 0;
			for(var i:int=0; i<requests.length; i++){
				var di:DataItem = requests[i];
				total += di.bytesLoaded;
			}
			return total;
		}
		public function get bytesTotal():Number{
			var total:Number = 0;
			for(var i:int=0; i<requests.length; i++){
				var di:DataItem = requests[i];
				total+=di.bytesTotal;
			}
			return total;
		}
		public function get progress():Number{			
			var per:Number = 0;
			for(var i:int=0; i<requests.length; i++){
				var di:DataItem = requests[i];
				if(di.bytesTotal>10){
					per += (di.bytesLoaded/di.bytesTotal)/length;
				}
			}		
			return per;
		}
		public function get length():Number {
			return requests.length;
		}
		public function get current():Number {
			return _current;
		}
		public function get loaded():Boolean{
			return (progress==1);
		}
		
	}
}