package com.flashdynamix.net {
	import flash.events.*;
	import flash.media.*;
	import flash.net.*;
	import flash.utils.Timer;
	
	import com.flashdynamix.events.CustomEvent;	

	public class FLVPlayer extends EventDispatcher {

		private var nc:NetConnection;
		private var ns:NetStream;

		private var timer:Timer;
		private var _video:Video;
		private var _playing:Boolean = false;
		private var _played:Boolean = false;
		private var _buffering:Boolean = true;
		private var _bufferTime:Number = 5;
		private var _duration:Number = 0;
		private var canSeekToEnd:Boolean = true;

		public function FLVPlayer(vid:Video) {
			nc = new NetConnection();
			nc.connect(null);

			ns = new NetStream(nc);
			ns.bufferTime = _bufferTime;
			ns.client = {onMetaData:onMeta};
			ns.addEventListener(NetStatusEvent.NET_STATUS, onStatus, false, 0, true);
			
			video = vid;
			
			timer = new Timer(100);
			timer.addEventListener(TimerEvent.TIMER, onUpdate, false, 0, true);
		}
		private function onMeta(data:Object):void {
			duration = Number(data.duration);
			canSeekToEnd = data.canSeekToEnd;
		}
		private function onUpdate(e:TimerEvent):void{
			
			if( !loaded ){
				if( (buffering && atBufferCeil) || (!buffering && atBufferFloor) ){
					_buffering = atBufferFloor;
					pause(buffering);
					dispatchEvent(new CustomEvent(Event.CHANGE));
				}
			} else if(loaded && buffering){
				_buffering = false;
				pause(buffering);
				dispatchEvent(new CustomEvent(Event.CHANGE));
			} else if(atEnd && !_played) {
				_played = true;
				dispatchEvent(new CustomEvent(Event.COMPLETE));
			}
		}
		public function load(url:String):void {
			
			_buffering = true;
			_played = false;
			_playing = true;
			_duration = 0;
			
			pause(buffering);
			reatach();
			ns.play(url);
			timer.start();
		}
		public function stop():void {
			_playing = false;
			_played = false;
			pause(true);
			ns.seek(0);
		}
		public function pause(paused:* = null):void {
			if(paused == null){
				_playing = !playing;
				ns.togglePause();
			} else if(paused == _playing){
				_playing = !paused;
				ns.togglePause();
			}
		}
		public function rewind():void {
			_played = false;
			ns.seek(0);
		}
		public function seek(per:Number):void{
			_played = false;
			var st:Number = Math.min(timeLoaded, int(per*duration));
			ns.seek(st);
		}
		public function detach():void{
			video.attachNetStream(null);
			video.clear();
		}
		public function reatach():void{
			video.attachNetStream(ns);
		}
		public function close():void{
			timer.stop();
			detach();
			ns.close();
		}
		public function dispose():void{
			close();
			
			detach();
			
			ns.removeEventListener(NetStatusEvent.NET_STATUS, onStatus);
			timer.removeEventListener(TimerEvent.TIMER, onUpdate);
			
			nc.close();
			
			nc = null;
			ns = null;
			timer = null;
			video = null;
		}
		private function onStatus(e:Event):void{
		}
		public function set soundOn(flag:Boolean):void {
			volume = (flag)?1: 0;
		}
		public function set volume(num:Number):void{
			ns.soundTransform = new SoundTransform(num);
		}
		public function get volume():Number {
			return ns.soundTransform.volume;
		}
		public function set duration(num:Number):void {
			_duration = num;
		}
		public function get duration():Number {
			return _duration;
		}
		public function get percentLoaded():Number {
			if(ns.bytesLoaded<=0 || ns.bytesTotal<=0) return 0;
			return Math.max(0, Math.min(1, ns.bytesLoaded / ns.bytesTotal) );
		}
		public function get loaded():Boolean{
			return (percentLoaded == 1);
		}
		public function get progress():Number {
			if(time<=0 || duration<=0) return 0;
			return Math.max(0, Math.min(1, time / duration));
		}
		public function get buffer():Number {
			return Math.max(0, Math.min(1, ns.bufferLength / ns.bufferTime));
		}
		public function get time():Number {
			return ns.time;
		}
		public function get timeLoaded():Number {
			return percentLoaded*duration;
		}
		public function get buffering():Boolean {
			return _buffering;
		}
		public function get playing():Boolean {
			return _playing;
		}
		public function set bufferTime(n:Number):void{
			_bufferTime = n;
			ns.bufferTime = _bufferTime;
		}
		public function get bufferTime():Number{
			return _bufferTime;
		}
		public function set video(v:Video):void{
			if(v == null) return;
			
			_video = v;
			video.attachNetStream(ns);
		}
		public function get video():Video{
			return _video;
		}
		public function get atEnd():Boolean{
			return ((duration-time) <=0.1 && duration>0);
		}
		private function get atBufferCeil():Boolean{
			return (ns.bufferLength > ns.bufferTime);
		}
		private function get atBufferFloor():Boolean{
			return (ns.bufferLength < 1);
		}
	}
}