﻿package com.flashdynamix.services {
	import flash.events.*;
	import flash.net.*;
	
	import com.flashdynamix.events.CustomEvent;
	import com.flashdynamix.net.*;
	import com.flashdynamix.services.YouTubeData;	

	public class YouTube extends EventDispatcher {

		private var dl : DataLoader;
		
		// This is the You Tube developer key you can get your own here http://code.google.com/apis/youtube/overview.html
		public static var clientKey : String = "ytapi-ShaneMcCartney-flashdynamix-vuj2k916-0";

		public static const FLVUrl : String = "http://www.youtube.com/get_video?video_id=";

		private static const APIUrl : String = "http://gdata.youtube.com/feeds/api/";
		private static const servicesDomain : String = "http://code.flashdynamix.com/YouTube/";
		private static const videoIdUrl : String = servicesDomain + "getVideoId.aspx";
		//private static const videoIdUrl:String = servicesDomain+"getVideoId.php";

		private static const VIDEOS_METHOD : String = "videos";
		private static const PLAYLISTS_METHOD : String = "playlists";
		private static const USERS_METHOD : String = "users";
		private static const UPLOADS_METHOD : String = "uploads";
		private static const FAVOURITES_METHOD : String = "favorites";
		private static const SUBSCRIPTION_METHOD : String = "subscriptions";
		private static const CONTACTS_METHOD : String = "contacts";

		public static const USERS : String = "user";
		public static const SEARCH : String = "search";
		public static const PLAYLISTS : String = "playlist";
		public static const FAVOURITES : String = "favorite";
		public static const SUBSCRIPTION : String = "subscription";
		public static const PROFILE : String = "profile";
		public static const CONTACTS : String = "contact";
		public static const VIDEOID : String = "videoId";

		
		public function YouTube() {
			dl = new DataLoader();
			dl.addEventListener(Event.COMPLETE, onLoaded);
			dl.addEventListener(ErrorEvent.ERROR, onError);
		}

		private function onLoaded(e : CustomEvent) : void {
			try {
				var di : DataItem = e.value as DataItem;
				
				var xml : XML = new XML(di.data);
				var media : Namespace = xml.namespace("media");
				
				var data : Object;
				
				switch(e.id) {
					case USERS :
						data = YouTubeData.videos(xml, xml.elements().media::group);
						break;
					case SEARCH :
						data = YouTubeData.videos(xml, xml.channel.item.media::group);
						break;
					case PLAYLISTS :
						data = YouTubeData.playlist(xml, xml.elements());
						break;
					case FAVOURITES :
						data = YouTubeData.videos(xml, xml.elements().media::group);
						break;
					case SUBSCRIPTION :
						data = YouTubeData.subscription(xml, xml.elements());
						break;
					case PROFILE :
						data = YouTubeData.profile(xml);
						break;
					case CONTACTS :
						data = YouTubeData.contacts(xml, xml.elements());
						break;
					default :
						data = xml;
					break;
				}
				
				dispatchEvent(new CustomEvent(Event.COMPLETE, e.id, data));
			} catch(error : Error) {
				dispatchEvent(new CustomEvent(ErrorEvent.ERROR, e.id));
			}
		}

		private function onError(e : CustomEvent) : void {
			dispatchEvent(new CustomEvent(ErrorEvent.ERROR, e.id));
		}
		
		public function videosForTag(tag : String, category : String = null, startIndex : Number = 1,maxResults : Number = 20, orderBy : String = "relevance", author : String = null, lr : String = null, restriction : String = null, time : String = null, racy : String = "exclude") : void {
			var ur : URLRequest = urlRequest;
			ur.url += VIDEOS_METHOD;
			ur.url += (category == null) ? "" : "/-/" + escape(category)+"/";
			ur.url += "?vq=" + escape(tag);
			ur.url += "&start-index=" + startIndex;
			ur.url += "&max-results=" + maxResults + "&";
			
			var uv : URLVariables = urlVars;
			uv.racy = racy;
			uv.orderby = orderBy;
			if(author) uv.author = author;
			if(lr) uv.lr = lr;
			if(restriction) uv.restriction = restriction;
			if(time) uv.time = time;
			ur.data = uv;
			
			loadData(ur, SEARCH);
		}

		public function videosForUser(userId : String = "default", startIndex : Number = 1,maxResults : Number = 20) : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId + "/" + UPLOADS_METHOD;
			ur.url += "?start-index=" + startIndex;
			ur.url += "&max-results=" + maxResults + "&";
			
			loadData(ur, USERS);
		}

		public function videosForUserPlaylist(userId : String) : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId + "/" + PLAYLISTS_METHOD;
			
			loadData(ur, PLAYLISTS);
		}

		public function videosForUserFavourites(userId : String = "default") : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId + "/" + FAVOURITES_METHOD;
			
			loadData(ur, FAVOURITES);
		}

		public function videosForUserSubscription(userId : String = "default") : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId + "/" + SUBSCRIPTION_METHOD;
			
			loadData(ur, SUBSCRIPTION);
		}

		public function userProfile(userId : String = "default") : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId;
			
			loadData(ur, PROFILE);
		}

		public function userContacts(userId : String = "default") : void {
			var ur : URLRequest = urlRequest;
			ur.url += USERS_METHOD + "/" + userId + "/" + CONTACTS_METHOD;
			
			loadData(ur, CONTACTS);
		}

		public function videoId(url : String) : void {
			var ur : URLRequest = new URLRequest(videoIdUrl);
			
			var uv : URLVariables = new URLVariables();
			uv.method = VIDEOID;
			uv.url = url;
			ur.data = uv;
			ur.method = URLRequestMethod.GET;
			
			loadData(ur, VIDEOID);
		}

		private function loadData(ur : URLRequest, id : String = null) : void {
			dl.load(ur, id);
		}

		private function get urlRequest() : URLRequest {
			var ur : URLRequest = new URLRequest(APIUrl);
			ur.method = URLRequestMethod.GET;
			return ur;
		}

		private function get urlVars() : URLVariables {
			var uv : URLVariables = new URLVariables();
			uv.client = clientKey;
			uv.alt = "rss";
			uv.format = 1;
			
			return uv;
		}
	}
}